import Input from "../src/input/index"

export default function tck(TypedDocument,file, debug=false, more=a=>a){
    if(!debug)
        console.debug=console.log=a=>a

    const data=require("fs").readFileSync(file)
    describe("event based reducer", ()=>{
        let editor=null

        function getBlocks(){
            const tables=editor.$("table")
            const ps=editor.$("paragraph").not(tables.find("paragraph"))
            return tables.add(ps)
        }

        function leaveOnlyFirst(a){
            const blocks=getBlocks()
            const first=editor.$().findFirst(a)
            blocks.not(first).toArray().forEach(id=>{
                editor.file.getNode(id).remove()
                editor.$(`#${id}`).remove()
            })
            expect(getBlocks().length).toBe(1)
            return first
        }

        beforeEach(()=>{
            Input.get=jest.fn(()=>TypedDocument)
            return Input.parse({data}).then(doc=>{
                const reducer=doc.buildReducer()
                const state=reducer()
                doc.onChange=jest.fn(function(){
                    editor=new TypedDocument.Reducer(...arguments)
                    editor.debug=debug
                    editor.isNumberingParagraph=jest.fn()
                    editor.paragraphHasIndentSetting=jest.fn()
                })
                reducer(state,{})
            }).catch(e=>{
                console.error(e)
            })
        })

        afterEach(()=>{
            editor=null
        })

        describe("seperation",()=>{
            describe('text',()=>{
                beforeEach(()=>{
                    expect(editor.$().findFirst('text').text().length>=5).toBe(true)
                })

                const seperate=(start,end)=>{
                    const texts=editor.$('text')
                    const first=texts.first()
                    const text=first.text()
                    const id=first.attr('id')
                    editor.cursorAt(id,start,id,end||text.length)
                    editor.seperateSelection()
                    return {textLength:texts.length, text}
                }
                
                it("seperate a text should create 2 more text",()=>{
                    const start=1, end=2
                    const {textLength,text}=seperate(start,end)
                    expect(editor.$('text').length).toBe(textLength+2)
                    const $target=editor.$target
                    expect($target.text()).toBe(text.substring(start,end))
                    const next=$target.forwardFirst("text")
                    expect(next.text()).toBe(text.substring(end,text.length))
                    const prev=$target.backwardFirst("text")
                    expect(prev.text()).toBe(text.substring(0,start))
                })
    
                it("seperate a text at beginning should create 1 more text forward",()=>{
                    const start=0, end=2
                    const {textLength,text}=seperate(start,end)
                    expect(editor.$('text').length).toBe(textLength+1)
                    const $target=editor.$target
                    expect($target.text()).toBe(text.substring(start,end))
                    const next=$target.forwardFirst("text")
                    expect(next.text()).toBe(text.substring(end,text.length))
                })

                it("seperate a text at end should create 1 more text backward",()=>{
                    const start=1
                    const {textLength,text}=seperate(start)
                    expect(editor.$('text').length).toBe(textLength+1)
                    const $target=editor.$target
                    expect($target.text()).toBe(text.substring(start))
                    const next=$target.backwardFirst("text")
                    expect(next.text()).toBe(text.substring(0,start))
                })

                it("seperate a whole text at end should not change anything",()=>{
                    const start=0
                    const {textLength,text}=seperate(start)
                    expect(editor.$('text').length).toBe(textLength)
                    const $target=editor.$target
                    expect($target.text()).toBe(text)
                })
            })

            describe("inline container",()=>{
                it("seperate whole inline container should not change anyting",()=>{
                    const container=editor.$().findFirst("paragraph * text").parent()
                    expect(container.length>0).toBe(true)
                    const id=container.attr('id')
                    editor.cursorAt(id,0, id,1)
                    const sel0=editor.selection
                    const next=editor.$target.forwardFirst(), prev=editor.$target.backwardFirst()
                    editor.seperateSelection()
                    expect(editor.selection).toMatchObject(sel0)
                    expect(editor.$target.forwardFirst().attr('id')).toBe(next.attr('id'))
                    expect(editor.$target.backwardFirst().attr('id')).toBe(prev.attr('id'))
                })
            })
        })

        describe("remove selection",()=>{
            const removeSelection=()=>editor.remove()
            it("remove text should not change structure",()=>{
                const texts=editor.$("text")
                const textLength=texts.length
                expect(textLength>0).toBe(true)
                const $first=texts.first()
                const id=$first.attr('id'), text=$first.text()
                expect(text.length>3).toBe(true)
                editor.cursorAt(id,1,id,2)
                removeSelection()
                expect(editor.$('text').length).toBe(textLength)
                expect(editor.selection).toMatchObject({start:{id,at:1}, end:{id,at:1}})
                expect(editor.$target.text()).toBe(text.substring(0,1)+text.substring(1+1))
            })

            it("remove whole text should not change structure",()=>{
                const texts=editor.$("text")
                const textLength=texts.length
                expect(textLength>0).toBe(true)
                const $first=texts.first()
                const id=$first.attr('id'), text=$first.text()
                expect(text.length>3).toBe(true)
                editor.cursorAt(id,0,id,text.length)
        
                removeSelection()
                expect(editor.$('text').length).toBe(textLength-1)
                expect(editor.$(`#${id}`).length).toBe(0)
            })

            it("remove inline container",()=>{
                const $container=editor.$().findFirst("paragraph * text").parent()
                expect($container.length>0).toBe(true)
                const id=$container.attr('id')
                editor.cursorAt(id,0,id,1)
                removeSelection()
                expect(editor.$(`#${id}`).length).toBe(0)
            })

            it("remove the only paragraph should leave an empty paragraph",()=>{
                if(!editor.onlyFlow)
                    return 
                const p=leaveOnlyFirst("paragraph")
                expect(p.length).toBe(1)
                expect(editor.$("paragraph").text().length>0).toBe(true)
                const id=p.attr('id')
                editor.cursorAt(id,0,id,1)
                removeSelection()
                expect(editor.$("paragraph").length).toBe(1)
                expect(editor.$("paragraph").text().length).toBe(0)
            })
        })

        describe("backspace",()=>{
            const backspace=()=>editor.backspace()
            it("backspace text should remove a char",()=>{
                const first=editor.$().findFirst("text")
                const len=first.text().length
                editor.cursorAt(first.attr('id'), 2)
                backspace()
                expect(first.text().length).toBe(len-1)
                expect(editor.selection.start).toMatchObject({id:first.attr('id'),at:1})
            })

            it("backspace at beginning of anything up to document should do anything",()=>{
                const first=leaveOnlyFirst("paragraph").findFirst("text")
                const text=first.text()
                editor.cursorAt(first.attr('id'),0)
                backspace()
                expect(first.text()).toBe(text)
            })

            it("backspace at beginning of text should remove prev's content",()=>{
                const first=editor.$().findFirst("text")
                const len=first.text().length
                expect(len>1).toBe(true)
                const second=first.forwardFirst("text")
                expect(first.attr('id')!=second.attr('id')).toBe(true)
                expect(second.closest("paragraph").attr('id')).toBe(first.closest("paragraph").attr("id"))
                editor.cursorAt(second.attr('id'),0)
                backspace()
                expect(first.text().length).toBe(len-1)
            })

            it("backspace at end of image should remove image",()=>{
                const images=editor.$("image")
                expect(images.length>0).toBe(true)
                const image=images.first()
                editor.cursorAt(image.attr('id'),1)
                backspace()
                expect(editor.$("image").length).toBe(images.length-1)
            })

            it("backspace at first up to to cell should do nothing",()=>{
                const table=editor.$().findFirst('table')
                expect(table.length).toBe(1)
                const first=table.findFirst("cell text")
                expect(first.length).toBe(1)
                editor.cursorAt(first.attr('id'),0)
                const selection=editor.selection
                backspace()
                expect(editor.selection).toMatchObject(selection)
            })

            it('backspace at beginning of paragraph should merge to prev paragraph',()=>{
                const ps=editor.$("paragraph")
                const $next=editor.$().findFirst("paragraph+paragraph")
                expect($next.length>0).toBe(true)
                const $p=$next.prev("paragraph")
                const lenP=$p.children().length, lenPNext=$next.children().length
                editor.cursorAt($next.attr('id'),0)
                editor.backspace()
                expect(editor.$("paragraph").length).toBe(ps.length-1)
                expect($p.children().length).toBe(lenP+lenPNext)
            })
        })

        describe("delete",()=>{
            it("delete at text should remove char at cursor,and cursor postion should not be changed",()=>{
                const first=editor.$().findFirst("text")
                const len=first.text().length
                editor.cursorAt(first.attr('id'), 2)
                editor.delete()
                expect(first.text().length).toBe(len-1)
                expect(editor.selection.start).toMatchObject({id:first.attr('id'),at:2})
            })

            it("delete at end of up to paragraph should merge next paragraph",()=>{
                const ps=editor.$("paragraph")
                const $next=editor.$().findFirst("paragraph+paragraph")
                expect($next.length>0).toBe(true)
                const $p=$next.prev("paragraph")
                const text=$p.findLast(editor.cursorable)
                const lenP=$p.children().length, lenPNext=$next.children().length
                expect(text.is("text")).toBe(true)
                editor.cursorAtEnd(text.attr('id'))
                editor.delete()
                expect(editor.$("paragraph").length).toBe(ps.length-1)
                expect($p.children().length).toBe(lenP+lenPNext)
            })

            it("delete at end of paragraph should merge next paragraph",()=>{
                const ps=editor.$("paragraph")
                const $next=editor.$().findFirst("paragraph+paragraph")
                expect($next.length>0).toBe(true)
                const $p=$next.prev("paragraph")
                const lenP=$p.children().length, lenPNext=$next.children().length
                editor.cursorAt($p.attr('id'),1)
                editor.delete()
                expect(editor.$("paragraph").length).toBe(ps.length-1)
                expect($p.children().length).toBe(lenP+lenPNext)
            })

            it("delete at end of last paragraph should do nothing",()=>{
                
            })

            it("delete at end of text should delete forward",()=>{
                const first=editor.$().findFirst("text")
                const firstText=first.text()
                editor.cursorAtEnd(first.attr('id'))
                const $next=editor.$target.forwardFirst(editor.cursorable)
                expect($next.attr('type')).toBe("text")
                const text=$next.text()
                editor.delete()
                expect(first.text()).toBe(firstText)
                expect($next.text().length).toBe(text.length-1)
                
                expect(editor.selection.start).toMatchObject({id:$next.attr('id'),at:0})
            })
        })

        describe("enter",()=>{
            const enter=()=>editor.enter()
            it("enter at text should split text up to paragraph",()=>{
                const ps=editor.$("paragraph")
                const first=editor.$().findFirst("text")
                expect(first.text().length>3).toBe(true)
                editor.cursorAt(first.attr('id'),2)
                enter()
                expect(editor.$("paragraph").length).toBe(ps.length+1)
                expect(editor.$().findFirst("text").text().length).toBe(2)
            })

            it("enter at first table cell and the table is the first of document should create new paragraph before table",()=>{
                if(!editor.onlyFlow)
                    return 
                expect(editor.$('table').length>0).toBe(true)
                const table=leaveOnlyFirst('table')
                const first=table.findFirst('cell paragraph text')
                expect(first.length).toBe(1)
                editor.cursorAt(first.attr('id'),0)
                enter()
                const blocks=getBlocks()
                expect(blocks.length).toBe(2)
                expect(editor.$target.forwardFirst(table).attr('id')).toBe(table.attr('id'))
            })

            it("enter at end of image should put image at first paragraph,cursor at second paragraph",()=>{
                const ps=editor.$('paragraph')
                const image=editor.$().findFirst("image")
                expect(image.length).toBe(1)
                editor.cursorAt(image.attr('id'),1)
                const $p=editor.$target.closest("paragraph")
                enter()
                expect(editor.$("paragraph").length).toBe(ps.length+1)
                expect(editor.$(image).length).toBe(1)
                expect(editor.selection.start.id).not.toBe(image.attr('id'))
                expect(editor.$target.closest("paragraph").findFirst(image).length).toBe(0)
            })

            it("should create new paragraph when entering at end of paragraph",()=>{
                const ps=editor.$('paragraph')
                const p=ps.eq(0)
                editor.cursorAt(p.attr('id'),1)
                editor.enter()
                expect(ps.length+1).toBe(editor.$('paragraph').length)
            })
        })

        describe("cursor move",()=>{
            it("forward in text should move cursor at next char",()=>{
                const first=editor.$().findFirst("text")
                editor.cursorAt(first.attr('id'),0)
                editor.forward()
                expect(editor.selection.start).toMatchObject({at:1})
            })

            it("forward at end of text should move to first of next cursorable",()=>{
                
            })

            it("forward at  end of  paragraph's last should go to end of paragraph",()=>{
                
            })
        })

        describe("structure",()=>{
            describe("section is flow container",()=>{
                it("section must have paragraph",()=>{

                })

                it("anchor should be in paragraph",()=>{

                })
            })

            describe("frame is flow and block container",()=>{
                it("frame can have paragraph",()=>{

                })

                it("frame can have anchor",()=>{

                })
            })

            describe("paragraph",()=>{

            })

            describe("text",()=>{
                it("must in paragraph",()=>{

                })
    
                it("can be in inline container in paragraph",()=>{
                    
                }) 
            })
            
            describe("anchor (position and wrap)",()=>{
                it("anchor in frame",()=>{

                })

                it("anchor in paragraph",()=>{

                })
            })

            describe("image is a shape",()=>{

            })
        })

        xdescribe("type",()=>{
            const x="x"
                
            describe("in text",()=>{
                it("at beginning should prepend, and cursor at 1",()=>{
                    const $=editor.$().findFirst('text')
                    const id=$.attr('id'), text=$.text()
                    editor.cursorAt(id,0)
                    editor.type(x)
                    expect($.text()).toBe(x+text)
                    expect(editor.cursor.at).toBe(1)
                })

                it("at end should append, and cursor at next of last",()=>{
                    const $=editor.$().findFirst('text')
                    const id=$.attr('id'), text=$.text()
                    editor.cursorAt(id,text.length)
                    editor.type(x)
                    expect($.text()).toBe(text+x)
                    expect(editor.cursor.at).toBe(text.length+1)
                })

                it("at middle should insert at i, and cursor at i+1",()=>{
                    const $=editor.$().findFirst('text')
                    const id=$.attr('id'), text=$.text()
                    editor.cursorAt(id,2)
                    editor.type(x)
                    expect($.text()).toBe(text.substring(0,2)+x+text.substring(2))
                    expect(editor.cursor.at).toBe(3)
                })
            })

            describe("in inline container",()=>{
                it("at begin to prepend to children",()=>{

                })

                it("at end to append to children",()=>{

                })

                it("empty container",()=>{

                })
            })

            describe("on image",()=>{

            })

            describe("at paragraph",()=>{
                it("end should append to last child",()=>{
                    const $p=editor.$().findFirst('paragraph')
                    editor.cursorAt($p.attr('id'),1,$p.attr('id'),1)
                    const text=$p.findLast('text').text(), children=$p.children()
                    editor.type(x)
                    expect(children.length).toBe($p.children().length)
                    expect($p.findLast('text').text()).toBe(text+x)
                    expect(editor.cursor.at).toBe(text.length+1+1)
                })

                it("end, but last child is not text, should create text at end",()=>{

                })

                it('begin should prepend to first child',()=>{
                    const $p=editor.$().findFirst('paragraph')
                    editor.cursorAt($p.attr('id'),0)
                    const text=$p.findFirst('text').text()
                    editor.type(x)
                    expect($p.findFirst('text').text()).toBe(x+text)
                    expect(editor.cursor.at).toBe(1)
                    expect(editor.cursor.id).toBe($p.findFirst('text').attr('id'))
                })

                it('begin,but first child is not text,  should create text at beginning',()=>{

                })

                it('of empty should create first text',()=>{
                    const $p=editor.$().findFirst('paragraph')
                })
            })
        })

        more(()=>editor)
    }) 
}

it("input event tck loaded",()=>{})